package serverP2P;

import java.util.Scanner;
import javafx.util.Pair;
import serverP2P.ServerManagementUDP;
import serverP2P.ServerManagementTCP;
import tools.Directories;
import tools.Logger;
import tools.HostItem;
import tools.TestPort;


/** Server only implementation
 * First argument of main method is port listened by the server, and is mandatory.
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class ServerP2P {
	private int port;
	private Directories directories;
	static private final String subdir = "seeded/";
	private Logger logger;
	private HostItem tracker;

	/** Constructor with portStr containing a port number.
	 * @param portStr	String containing port number of listening.
	 */
	public ServerP2P(String portStr, String trackerHostname, String trackerPortStr) {
		port = Integer.valueOf(Integer.parseInt(portStr));
		tracker = new HostItem(trackerHostname, Integer.valueOf(Integer.parseInt(trackerPortStr)));
		directories = new Directories("P2P_JAVA_PROJECT_SERVER_" + port);
		directories.createSubdir(subdir);
		logger = new Logger(directories.getDataHomeDirectory() + "server.log");
		System.out.println("Server will listen on port " + port + " and serve files from " + directories.getDataHomeDirectory() + subdir);
		Scanner scanner = new Scanner(System.in);
		directories.askOpenDataHomeDirectory(subdir, scanner);
		scanner.close();
	}

	/** Main program entry point
	 * first parameter is port number and is mandatory
	 * to test, run with: java -ea serverP2P.ServerP2P
	 * @param args	parameters
	 */
		public static void main(String [] args) {
			final String defaultHostname = "localhost";
			final String defaultPortServer = "7070";
			final String defaultPortTracker = "6969";
			String hostnameServer = "";
			String portServer = "";
			String hostnameTracker = "";
			String portTracker = "";
			Scanner scanner = new Scanner(System.in);
			TestPort testPort = new TestPort();

			if ((args.length != 5) && (args.length != 0)){
				System.out.println("usage : java serveurP2P.ServeurP2P (interactive) or java serveurP2P.ServeurP2P -- <serveurHOSTNAME> <serveurPORT> <trackerHOSTNAME> <trackerPORT>");
			}
			else{
				if(args.length == 5){
					hostnameServer = args[1];
					portServer = args[2];
					hostnameTracker = args[3];
					portTracker = args[4];
				}
				else{
					System.out.println("Server, enter hostname to bind (default = localhost): ");
					hostnameServer = scanner.nextLine();
					if(hostnameServer.equals("")){
						hostnameServer = defaultHostname;
						System.out.println("using default hostname : " + hostnameServer);
					}
					System.out.println("enter port (default = 7070): ");
					portServer = scanner.nextLine();
					if(portServer.equals("")){
						portServer = defaultPortServer;
						System.out.println("using default port : " + portServer);
					}
					System.out.println("enter hostname of tracker (default = localhost): ");
					hostnameTracker = scanner.nextLine();
					if(hostnameTracker.equals("")){
						hostnameTracker = defaultHostname;
						System.out.println("tracker default hostname : " + hostnameTracker);
					}
					System.out.println("enter tracker's port (default = 6969): ");
					portTracker = scanner.nextLine();
					if(portTracker.equals("")){
						portTracker = defaultPortTracker;
						System.out.println("using default port : " + portTracker);
					}
				}

				System.out.println("using hostname : " + hostnameServer);
				Pair<Integer, String> PtestPortServer = testPort.testPortServer(Integer.parseInt(portServer));
				if(PtestPortServer.getKey() == 0){
					System.out.println("using port : " + portServer);
				}
				else {
					System.out.println(PtestPortServer.getValue());
					portServer = defaultPortServer;
				}
				System.out.println("tracker hostname : " + hostnameTracker);

				Pair<Integer, String> PtestPortTracker = testPort.testPortTracker(Integer.parseInt(portTracker));
				if(PtestPortTracker.getKey() == 0){
					System.out.println("using port : " + portTracker);
				}
				else {
					System.out.println(PtestPortTracker.getValue());
					portTracker = defaultPortTracker;
				}

				ServerP2P s = new ServerP2P(portServer, hostnameTracker, portTracker);
				ServerManagementUDP smudp = new ServerManagementUDP(s.directories.getDataHomeDirectory() + subdir, hostnameServer, s.port, s.logger, s.tracker);
				ServerManagementTCP smtcp = new ServerManagementTCP(s.directories.getDataHomeDirectory() + subdir, hostnameServer, s.port, s.logger, s.tracker);
				Thread tudp = new Thread(smudp);
				tudp.setName("server UDP P2P-JAVA-PROJECT");
				tudp.start();
				Thread ttcp = new Thread(smtcp);
				ttcp.setName("server TCP P2P-JAVA-PROJECT");
				ttcp.start();
			}
		}
}
